﻿using Microsoft.EntityFrameworkCore;
using RestaurantContracts.BindingModels;
using RestaurantContracts.SearchModels;
using RestaurantContracts.StoragesContracts;
using RestaurantContracts.ViewModels;
using RestaurantDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RestaurantDatabaseImplement.Implements
{
    public class ProductStorage : IProductStorage
    {
        public List<ProductViewModel> GetFullList()
        {
            using var context = new RestaurantDatabase();
            return context.Products
            .Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<ProductViewModel> GetFilteredList(ProductSearchModel model)
        {
            using var context = new RestaurantDatabase();
            if (model.Id.HasValue)
            {
                return context.Products
                    .Include(x => x.Components)
                    .ThenInclude(x => x.Component)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.Type != null)
            {
                return context.Products
                    .Include(x => x.Components)
                    .ThenInclude(x => x.Component)
                    .Where(x => x.Type == model.Type)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }      
            else
            {
                return new();
            }
        }
        public ProductViewModel? GetElement(ProductSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Type) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new RestaurantDatabase();
            return context.Products
             .Include(x => x.Components)
             .ThenInclude(x => x.Component)
             .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Type) &&
            x.Type == model.Type) ||
            (model.Id.HasValue && x.Id ==
            model.Id))
            ?.GetViewModel;
        }
        public ProductViewModel? Insert(ProductBindingModel model)
        {
            using var context = new RestaurantDatabase();
            var newProduct = Product.Create(context, model);
            if (newProduct == null)
            {
                return null;
            }
            context.Products.Add(newProduct);
            context.SaveChanges();
            return newProduct.GetViewModel;
        }
        public ProductViewModel? Update(ProductBindingModel model)
        {
            using var context = new RestaurantDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var product = context.Products.FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (product == null)
                {
                    return null;
                }
                product.Update(model);
                context.SaveChanges();
                product.UpdateComponents(context, model);
                transaction.Commit();
                return product.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public ProductViewModel? Delete(ProductBindingModel model)
        {
            using var context = new RestaurantDatabase();
            var element = context.Products
            .Include(x => x.Components)
            .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Products.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
