﻿using RestaurantContracts.BusinessLogicsContracts;
using RestaurantContracts.ViewModels;
using RestaurantDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RestaurantView
{
    public partial class FormComponentProvider : Form
    {
        private readonly List<ProviderViewModel>? _list;

        public int Id
        {
            get
            {
                return Convert.ToInt32(comboBoxProviders.SelectedValue);
            }
            set
            {
                comboBoxProviders.SelectedValue = value;
            }
        }

        public IProviderModel? ProviderModel
        {
            get
            {
                if (_list == null)
                {
                    return null;
                }
                foreach (var elem in _list)
                {
                    if (elem.Id == Id)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }
        public int Count
        {
            get { return Convert.ToInt32(textBoxCount.Text); }
            set { textBoxCount.Text = value.ToString(); }
        }

        public DateTime Date
        {
            get { return DateTime.Now; }
            set { dateTimePicker.Value = value; }
        }

        public FormComponentProvider(IProviderLogic logic)
        {
            InitializeComponent();
            _list = logic.ReadList(null);
            if (_list != null)
            {
                comboBoxProviders.DisplayMember = "Name";
                comboBoxProviders.ValueMember = "Id";
                comboBoxProviders.DataSource = _list;
                comboBoxProviders.SelectedItem = null;
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxProviders.SelectedValue == null)
            {
                MessageBox.Show("Выберите компонент", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            DialogResult = DialogResult.OK;
            Close();
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
